package fr.eurecom.ratcom.util;


import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;

import java.io.RandomAccessFile;

/**
 * Title: FileHandler Description: Handle file things, like open, create
 * directory etc...
 * 
 * @Copyright: Copyright (c) 2006
 * @Company: Haiku project
 * 
 * @author Daniel Camara
 * @version 1.0
 */

public class FileHandler {
	/**
	 * Save an string in a new file. If the file already exists it will be
	 * overwrited
	 * 
	 * @param fileName -
	 *            Name of the file
	 * @param text -
	 *            what will be saved in the new file
	 */
	public static void saveOnFile(String fileName, String text)
			throws IOException {
		saveOnFile(fileName, text, false); // File
	} // saveOnFile

	public static String SEPARATOR = System.getProperty("file.separator");

	/**
	 * Save an string in a new file. If the file already exists it will be
	 * overwrited if the append parameter is true
	 * 
	 * @param fileName -
	 *            Name of the file
	 * @param text -
	 *            what will be saved in the new file
	 * @param append -
	 *            if the text should be appended to an existing file or if
	 *            it should be overwritten
	 */
	public static void saveOnFile(String fileName, String text, boolean append)
			throws IOException {
		FileWriter fw = new FileWriter(fileName, append); // File

		fw.write(text + "\n");

		fw.close();
	} // saveOnFile
	

	/**
	 * Creates a directory structure if it doesn't already exists
	 * 
	 * @param directories -
	 *            List of directories
	 */
	public static void createsDirectoryStructure(ArrayList<String> directories)
			throws IOException {

		File fd;

		// Creates the list of directories passed as parameter
		int nDirectories = directories.size();
		for (int i = 0; i < nDirectories; i++) {
			fd = new File(((String) directories.get(i)));
			fd.mkdir();
		}

	} // saveOnFile


	  /**
	   * Gest a vector of directories and returns a list of strings with their
	   * names
	   * 
	   * @param dirs
	   * @return
	   */
	  private static String[] filesToString(Vector<File> dirs) {
		int size = dirs.size();
		String[] s = new String[size];
		for(int i=0;i<size;i++)
			s[i]=((File)dirs.get(i)).getAbsolutePath();
		
		return s;
	}

	
	/**
	 * Reads a file and returns it as string
	 * 
	 * @param fileName -
	 *            The name of the file to read
	 * @return A string with the file content or null if something goes wrong
	 */
	public static int simpleOccurrences(String fileName, String toSearch) {

		String content = FileHandler.readFile(fileName.toLowerCase());
		
		return (content.toLowerCase().split(toSearch.toLowerCase()).length -1);
	}

	/**
	 * Reads a file and returns it as string
	 * 
	 * @param fileName -
	 *            The name of the file to read
	 * @return A string with the file content or null if something goes wrong
	 */
	public static String readFile(String fileName) {
		StringBuffer fileContent = new StringBuffer();

		try {
			BufferedReader br = new BufferedReader(new InputStreamReader(
					new FileInputStream(fileName)));
			String line = "";
			while ((line = br.readLine()) != null) {
				fileContent.append(line.trim() + " ");
			}
		} catch (Throwable e) {
			e.printStackTrace();
			return "";
		}

		return fileContent.toString();
	}
	
	/**
	 * Reads a file and returns it as string
	 * 
	 * @param fileName -
	 *            The name of the file to read
	 * @return A string with the file content or null if something goes wrong
	 */
	public static String readFileUTF8(String fileName) {
		StringBuffer fileContent = new StringBuffer();

		try {
			BufferedReader br = new BufferedReader(new InputStreamReader(
					new FileInputStream(fileName), "UTF-8"));
			String line = "";
			while ((line = br.readLine()) != null) {
				fileContent.append(line.trim() + " ");
			}
		} catch (Throwable e) {
			e.printStackTrace();
			return "";
		}

		return fileContent.toString();
	}
	/**
	 * Reads a file and returns it as string
	 * 
	 * @param fileName -
	 *            The name of the file to read
	 * @return A string with the file content or null if something goes wrong
	 */
	public static String readFileWithLines(String fileName) {
		StringBuffer fileContent = new StringBuffer();

		try {
			BufferedReader br = new BufferedReader(new InputStreamReader(
					new FileInputStream(fileName)));
			String line = "";
			while ((line = br.readLine()) != null) {
				fileContent.append(line.trim() + "\n");
			}
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}

		return fileContent.toString();
	}

	 /**
	   * Verify if the file name passed as parameter corresponds to an existent
	   * file on disk. If so returns true
	   * 
	   * @param fileNameNotFound
	   * @return
	   */
	  public static boolean verifyFileOnDisk(String fileName) {

	    // if the file already exists
	    if ((new File(fileName)).exists()) {
	      return true;
	    }
	    return false;
	  }

	/**
	 * Copy a file from one place to another.
	 * 
	 * @param inputFile -
	 *            the file to be copyed complete name
	 * @param outputFile -
	 *            the name and location of the new file
	 * 
	 * @param boolean -
	 *            true if everything is OK and false if not
	 */
	public static boolean copyFile(String inputFile, String outputFile) {

		// if the source and destination are the same
		if (inputFile.equals(outputFile))
			return true;

		// copy file
		try {
			RandomAccessFile fileReader = new RandomAccessFile(inputFile, "r");

			RandomAccessFile fileWriter = new RandomAccessFile(outputFile, "rw");

			int ch = 0;

			// reads the input file passing every thing to the otput file
			do {
				ch = fileReader.read();
				fileWriter.write(ch);
			} while (ch >= 0); // do while exists bytes at input file

			// Close files
			fileReader.close();
			fileWriter.close();

		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}

		return true;
	}

	/**
	 * Retrieves the informations about the photos from file. If any problem
	 * occurs it returns an empty ArrayList
	 * 
	 * @param photoList -
	 *            The list of information in ImageData format.
	 * @param fileName -
	 *            The name of file to save the information
	 * 
	 * @return boolean - true if everything was OK and false if not
	 */
	// public static ArrayList retrievesImageDataFromFile(String fileName){
	// ArrayList photoList = new ArrayList();
	//
	// try{
	// RandomAccessFile fr = new RandomAccessFile(fileName, "r");
	//
	// int numberOfPhotos = photoList.size();
	// ImageData photo;
	// String name;
	//
	// while((name = fr.readLine())!=null){
	// photo = new ImageData(name,
	// fr.readLine(), //directory
	// fr.readLine(), //title
	// fr.readLine(), //title style
	// fr.readLine(), //title color
	// fr.readLine(), //caption
	// fr.readLine(), // caption style
	// fr.readLine()); // caption color
	// photo.linkName = fr.readLine();//link name
	// photo.thumbnailName = fr.readLine();//thumbnail name
	// fr.readLine(); //separator
	// photoList.add(photo);
	// }
	// fr.close();
	// }catch(Exception e){
	// return new ArrayList();
	// }
	//
	// return photoList;
	// }
	/**
	 * Returns the main file name, with out extension
	 * 
	 * @param name -
	 *            the file name
	 * 
	 * @return String the main file name
	 */
	public static String mainFileName(String name) {
		StringTokenizer token = new StringTokenizer(name, ".");
		StringBuffer mainName = new StringBuffer();

		int tokensNumber = token.countTokens();
		int i = 0;

		// remount the string with out the extension
		do {
			mainName.append(token.nextToken() + ".");
			i++;
		} while (i < (tokensNumber - 1));

		if (mainName.length() > 0)
			mainName.deleteCharAt(mainName.length() - 1);

		return mainName.toString();
	}

	 /**
	   * Reads a file and return an array of strings where each string has one line
	   * of the file
	   * 
	   * @param fileNameNotFound -
	   *          The name of the file to read
	   * @return A string vector with the file content or null if something goes
	   *         wrong
	   */
	  public static String[] readLinedFile(String fileName)
	  {

	    Vector lines = readLinedFileVector(fileName);

	    if(lines == null)
	    	return null;
	    // creates the string vector
	    String[] stringArrayType = new String[lines.size()];
	    String[] stringArray = (String[]) lines.toArray(stringArrayType);

	    return stringArray;
	  }

	  /**
	   * Reads a file and return an array of strings where each string has one line
	   * of the file
	   * 
	   * @param fileNameNotFound -
	   *          The name of the file to read
	   * @return A string vector with the file content or null if something goes
	   *         wrong
	   */
	  public static Vector<String> readLinedFileVector(String fileName)
	  {

	    try
	    {
	      BufferedReader br = new BufferedReader(new InputStreamReader(
	          new FileInputStream(fileName)));
	      return readLinedFileVector(br);
	    } catch (Exception e)
	    {
	      e.printStackTrace();
	      return null;
	    }
	  }

	  /**
	   * Reads a file and return an array of strings where each string has one line
	   * of the file
	   * 
	   * @param BufferedReader - the buffer of the file to read
	   * @return A string vector with the file content or null if something goes
	   *         wrong
	   */
	  public static Vector<String> readLinedFileVector(BufferedReader br)
	  {
	    Vector<String> lines = new Vector<String>();

	    try
	    {
	      String line = "";
	      while ((line = br.readLine()) != null)
	      {
	        lines.add(line.trim());
	      }
	    } catch (Exception e)
	    {
	      e.printStackTrace();
	      return null;
	    }
	  
	    return lines;
	  }
	public static void main(String[] args) {
//		System.out.println(occurrences("D:\\Work\\workspace\\ConferencesPage\\", "xml","2008"));
		
		System.out.println(mainFileName("teste.fim"));
		System.out.println(mainFileName("teste33"));
		System.out.println(mainFileName("vai.catar.coquinho.com"));
		System.out.println(mainFileName(".vai"));
		System.out.println(mainFileName(".vai.dar.caca"));

	}
}
